package org.ow2.weblab.core.model;

/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2010 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */

import java.net.URI;

import org.ow2.weblab.core.annotator.BaseAnnotator;
import org.ow2.weblab.rdf.Value;
import org.ow2.weblab.rdf.XMLParser;

 /**
  * WebLab Exchange model v1.2 - August 2010
  * This version includes annotatable object of the model i.e. Resources and their relationships.
  * It also provides some properties to be used in annotations.
  * 
  * WEBLAB: Service oriented integration platform for media mining and intelligence applications Copyright (C) 2004 - 2010 CASSIDIAN
  */
public class WModelAnnotator extends BaseAnnotator {

	protected static final String PREFIX = "wmodel";
	protected static final URI URI = XMLParser.buildURI("http://weblab.ow2.org/core/1.2/ontology/model#");

	public WModelAnnotator(URI subject, PieceOfKnowledge pieceOfKnowledge){
		super(subject,pieceOfKnowledge);
	}

	public WModelAnnotator(Resource resource){
		super(resource);
	}

	/**
	 * Read the The end value of a LinearSegment or a TemporalSegment. 
	 * 
	 * @return an Integer
	 */
	public Value<Integer> readEndAt(){
		return applyOperator(Operator.READ, null,URI, END_AT, Integer.class, null);
	}


	/**
	 * Write the The end value of a LinearSegment or a TemporalSegment.
	 * @param value an Integer
	 */
	public void writeEndAt(Integer value){
		applyOperator(Operator.WRITE, PREFIX,URI, END_AT, Integer.class, value);
	}


	/**
	 * Read the The start value of a LinearSegment or a TemporalSegment. 
	 * 
	 * @return an Integer
	 */
	public Value<Integer> readStartAt(){
		return applyOperator(Operator.READ, null,URI, START_AT, Integer.class, null);
	}


	/**
	 * Write the The start value of a LinearSegment or a TemporalSegment.
	 * @param value an Integer
	 */
	public void writeStartAt(Integer value){
		applyOperator(Operator.WRITE, PREFIX,URI, START_AT, Integer.class, value);
	}


	/**
	 * Read the Links a Resource to one of its Annotation. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readAnnotation(){
		return applyOperator(Operator.READ, null,URI, HAS_ANNOTATION, URI.class, null);
	}


	/**
	 * Write the Links a Resource to one of its Annotation.
	 * @param value an URI
	 */
	public void writeAnnotation(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_ANNOTATION, URI.class, value);
	}


	/**
	 * Read the Links a SpatialSegment to one of its Coordinate. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readCoordinate(){
		return applyOperator(Operator.READ, null,URI, HAS_COORDINATE, URI.class, null);
	}


	/**
	 * Write the Links a SpatialSegment to one of its Coordinate.
	 * @param value an URI
	 */
	public void writeCoordinate(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_COORDINATE, URI.class, value);
	}


	/**
	 * Read the Links a Resource to one of its LowLevelDescriptor. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readLowLevelDescriptor(){
		return applyOperator(Operator.READ, null,URI, HAS_LOW_LEVEL_DESCRIPTOR, URI.class, null);
	}


	/**
	 * Write the Links a Resource to one of its LowLevelDescriptor.
	 * @param value an URI
	 */
	public void writeLowLevelDescriptor(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_LOW_LEVEL_DESCRIPTOR, URI.class, value);
	}


	/**
	 * Read the Links a ResultSet to its PieceOfKnowledge. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readPok(){
		return applyOperator(Operator.READ, null,URI, HAS_POK, URI.class, null);
	}


	/**
	 * Write the Links a ResultSet to its PieceOfKnowledge.
	 * @param value an URI
	 */
	public void writePok(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_POK, URI.class, value);
	}


	/**
	 * Read the Links a MediaUnit to one of its Segment. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readSegment(){
		return applyOperator(Operator.READ, null,URI, HAS_SEGMENT, URI.class, null);
	}


	/**
	 * Write the Links a MediaUnit to one of its Segment.
	 * @param value an URI
	 */
	public void writeSegment(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_SEGMENT, URI.class, value);
	}


	/**
	 * Read the Links a resource to its parent ComposedResource 
	 * 
	 * @return an URI
	 */
	public Value<URI> readInComposedResource(){
		return applyOperator(Operator.READ, null,URI, IN_COMPOSED_RESOURCE, URI.class, null);
	}


	/**
	 * Write the Links a resource to its parent ComposedResource
	 * @param value an URI
	 */
	public void writeInComposedResource(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IN_COMPOSED_RESOURCE, URI.class, value);
	}


	/**
	 * Read the Links a MediaUnit to its parent Document. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readInDocument(){
		return applyOperator(Operator.READ, null,URI, IN_DOCUMENT, URI.class, null);
	}


	/**
	 * Write the Links a MediaUnit to its parent Document.
	 * @param value an URI
	 */
	public void writeInDocument(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IN_DOCUMENT, URI.class, value);
	}


	/**
	 * Read the Links a resource to its parent ResultSet. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readInResultSet(){
		return applyOperator(Operator.READ, null,URI, IN_RESULT_SET, URI.class, null);
	}


	/**
	 * Write the Links a resource to its parent ResultSet.
	 * @param value an URI
	 */
	public void writeInResultSet(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IN_RESULT_SET, URI.class, value);
	}


	/**
	 * Read the Links a Document to one of its sub MediaUnit. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readComposedByMediaUnit(){
		return applyOperator(Operator.READ, null,URI, IS_COMPOSED_BY_MEDIA_UNIT, URI.class, null);
	}


	/**
	 * Write the Links a Document to one of its sub MediaUnit.
	 * @param value an URI
	 */
	public void writeComposedByMediaUnit(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_COMPOSED_BY_MEDIA_UNIT, URI.class, value);
	}


	/**
	 * Read the Links a ComposedQuery one of its Query. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readComposedByQuery(){
		return applyOperator(Operator.READ, null,URI, IS_COMPOSED_BY_QUERY, URI.class, null);
	}


	/**
	 * Write the Links a ComposedQuery one of its Query.
	 * @param value an URI
	 */
	public void writeComposedByQuery(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_COMPOSED_BY_QUERY, URI.class, value);
	}


	/**
	 * Read the Links a ComposedResource to one of its Resource. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readComposedByResource(){
		return applyOperator(Operator.READ, null,URI, IS_COMPOSED_BY_RESOURCE, URI.class, null);
	}


	/**
	 * Write the Links a ComposedResource to one of its Resource.
	 * @param value an URI
	 */
	public void writeComposedByResource(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_COMPOSED_BY_RESOURCE, URI.class, value);
	}


	/**
	 * Read the Links a ResultSet to one of its Resource. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readComposedByResourceResultSet(){
		return applyOperator(Operator.READ, null,URI, IS_COMPOSED_BY_RESOURCE_RESULT_SET, URI.class, null);
	}


	/**
	 * Write the Links a ResultSet to one of its Resource.
	 * @param value an URI
	 */
	public void writeComposedByResourceResultSet(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_COMPOSED_BY_RESOURCE_RESULT_SET, URI.class, value);
	}


	protected static final String END_AT = "endAt";
	protected static final String START_AT = "startAt";
	protected static final String HAS_ANNOTATION = "hasAnnotation";
	protected static final String HAS_COORDINATE = "hasCoordinate";
	protected static final String HAS_LOW_LEVEL_DESCRIPTOR = "hasLowLevelDescriptor";
	protected static final String HAS_POK = "hasPok";
	protected static final String HAS_SEGMENT = "hasSegment";
	protected static final String IN_COMPOSED_RESOURCE = "inComposedResource";
	protected static final String IN_DOCUMENT = "inDocument";
	protected static final String IN_RESULT_SET = "inResultSet";
	protected static final String IS_COMPOSED_BY_MEDIA_UNIT = "isComposedByMediaUnit";
	protected static final String IS_COMPOSED_BY_QUERY = "isComposedByQuery";
	protected static final String IS_COMPOSED_BY_RESOURCE = "isComposedByResource";
	protected static final String IS_COMPOSED_BY_RESOURCE_RESULT_SET = "isComposedByResourceResultSet";

}

