package org.ow2.weblab.core.model.processing;

/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2010 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */


import java.net.URI;
import java.util.Date;
import org.ow2.weblab.rdf.Value;
import org.ow2.weblab.rdf.XMLParser;
import org.ow2.weblab.core.model.*;
import org.ow2.weblab.core.annotator.*;

 /**
  * WebLab Exchange model used for resources processing v0.3.
  *  
  * This version includes relationships used for WebLab Resources processing.
  * 
  * WEBLAB: Service oriented integration platform for media mining and intelligence applications Copyright (C) 2004 - 2010 CASSIDIAN
  */
public class WProcessingAnnotator extends BaseAnnotator {

	protected static final String PREFIX = "wprocessing";
	protected static final URI URI = XMLParser.buildURI("http://weblab.ow2.org/core/1.2/ontology/processing#");

	public WProcessingAnnotator(URI subject, PieceOfKnowledge pieceOfKnowledge){
		super(subject,pieceOfKnowledge);
	}

	public WProcessingAnnotator(Resource resource){
		super(resource);
	}

	/**
	 * Read the Media unit can be ignored by a service 
	 * 
	 * @return a Boolean
	 */
	public Value<Boolean> readCanBeIgnored(){
		return applyOperator(Operator.READ, null,URI, CAN_BE_IGNORED, Boolean.class, null);
	}


	/**
	 * Write the Media unit can be ignored by a service
	 * @param value a Boolean
	 */
	public void writeCanBeIgnored(Boolean value){
		applyOperator(Operator.WRITE, PREFIX,URI, CAN_BE_IGNORED, Boolean.class, value);
	}


	/**
	 * Read the Value that corresponds to the proximity between the two clusters implied in a cluster link. 
	 * 
	 * @return a Double
	 */
	public Value<Double> readClusterProximity(){
		return applyOperator(Operator.READ, null,URI, HAS_CLUSTER_PROXIMITY, Double.class, null);
	}


	/**
	 * Write the Value that corresponds to the proximity between the two clusters implied in a cluster link.
	 * @param value a Double
	 */
	public void writeClusterProximity(Double value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_CLUSTER_PROXIMITY, Double.class, value);
	}


	/**
	 * Read the Gives the date when the Document was gathered. 
	 * 
	 * @return a Date
	 */
	public Value<Date> readGatheringDate(){
		return applyOperator(Operator.READ, null,URI, HAS_GATHERING_DATE, Date.class, null);
	}


	/**
	 * Write the Gives the date when the Document was gathered.
	 * @param value a Date
	 */
	public void writeGatheringDate(Date value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_GATHERING_DATE, Date.class, value);
	}


	/**
	 * Read the Gives the name of the original file gathered. 
	 * 
	 * @return a String
	 */
	public Value<String> readOriginalFileName(){
		return applyOperator(Operator.READ, null,URI, HAS_ORIGINAL_FILE_NAME, String.class, null);
	}


	/**
	 * Write the Gives the name of the original file gathered.
	 * @param value a String
	 */
	public void writeOriginalFileName(String value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_ORIGINAL_FILE_NAME, String.class, value);
	}


	/**
	 * Read the Gives the number of bytes of the original file gathered. 
	 * 
	 * @return a Long
	 */
	public Value<Long> readOriginalFileSize(){
		return applyOperator(Operator.READ, null,URI, HAS_ORIGINAL_FILE_SIZE, Long.class, null);
	}


	/**
	 * Write the Gives the number of bytes of the original file gathered.
	 * @param value a Long
	 */
	public void writeOriginalFileSize(Long value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_ORIGINAL_FILE_SIZE, Long.class, value);
	}


	/**
	 * Read the Gives the stem value for this text. 
	 * 
	 * @return a String
	 */
	public Value<String> readStem(){
		return applyOperator(Operator.READ, null,URI, HAS_STEM, String.class, null);
	}


	/**
	 * Write the Gives the stem value for this text.
	 * @param value a String
	 */
	public void writeStem(String value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_STEM, String.class, value);
	}


	/**
	 * Read the Whether or not is the instance extracted a candidate instance. 
	 * 
	 * @return a Boolean
	 */
	public Value<Boolean> readCandidate(){
		return applyOperator(Operator.READ, null,URI, IS_CANDIDATE, Boolean.class, null);
	}


	/**
	 * Write the Whether or not is the instance extracted a candidate instance.
	 * @param value a Boolean
	 */
	public void writeCandidate(Boolean value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_CANDIDATE, Boolean.class, value);
	}


	/**
	 * Read the Is exposed on a given protocol at a given address (for instance points to an
	 *  URL). 
	 * 
	 * @return a String
	 */
	public Value<String> readExposedAs(){
		return applyOperator(Operator.READ, null,URI, IS_EXPOSED_AS, String.class, null);
	}


	/**
	 * Write the Is exposed on a given protocol at a given address (for instance points to an
	 *  URL).
	 * @param value a String
	 */
	public void writeExposedAs(String value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_EXPOSED_AS, String.class, value);
	}


	/**
	 * Read the Links a resource to the service that has modified this resource. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readBeenUpdatedBy(){
		return applyOperator(Operator.READ, null,URI, HAS_BEEN_UPDATED_BY, URI.class, null);
	}


	/**
	 * Write the Links a resource to the service that has modified this resource.
	 * @param value an URI
	 */
	public void writeBeenUpdatedBy(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_BEEN_UPDATED_BY, URI.class, value);
	}


	/**
	 * Read the 
	 * 
	 * @return an URI
	 */
	public Value<URI> readNativeContent(){
		return applyOperator(Operator.READ, null,URI, HAS_NATIVE_CONTENT, URI.class, null);
	}


	/**
	 * Write the 
	 * @param value an URI
	 */
	public void writeNativeContent(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_NATIVE_CONTENT, URI.class, value);
	}


	/**
	 * Read the Links a media unit to its normalised content. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readNormalisedContent(){
		return applyOperator(Operator.READ, null,URI, HAS_NORMALISED_CONTENT, URI.class, null);
	}


	/**
	 * Write the Links a media unit to its normalised content.
	 * @param value an URI
	 */
	public void writeNormalisedContent(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, HAS_NORMALISED_CONTENT, URI.class, value);
	}


	/**
	 * Read the Link a ClusterLink with it the two clusters composants. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readComposedByCluster(){
		return applyOperator(Operator.READ, null,URI, IS_COMPOSED_BY_CLUSTER, URI.class, null);
	}


	/**
	 * Write the Link a ClusterLink with it the two clusters composants.
	 * @param value an URI
	 */
	public void writeComposedByCluster(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_COMPOSED_BY_CLUSTER, URI.class, value);
	}


	/**
	 * Read the Links a media unit result of an automatic (translation, summarisation,
	 *  transcription) process from a source media unit. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readGeneratedFrom(){
		return applyOperator(Operator.READ, null,URI, IS_GENERATED_FROM, URI.class, null);
	}


	/**
	 * Write the Links a media unit result of an automatic (translation, summarisation,
	 *  transcription) process from a source media unit.
	 * @param value an URI
	 */
	public void writeGeneratedFrom(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_GENERATED_FROM, URI.class, value);
	}


	/**
	 * Read the Link a WebLab Document to a Cluster 
	 * 
	 * @return an URI
	 */
	public Value<URI> readInCluster(){
		return applyOperator(Operator.READ, null,URI, IS_IN_CLUSTER, URI.class, null);
	}


	/**
	 * Write the Link a WebLab Document to a Cluster
	 * @param value an URI
	 */
	public void writeInCluster(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_IN_CLUSTER, URI.class, value);
	}


	/**
	 * Read the Link a cluster with a ClusterLink. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readInClusterLink(){
		return applyOperator(Operator.READ, null,URI, IS_IN_CLUSTER_LINK, URI.class, null);
	}


	/**
	 * Write the Link a cluster with a ClusterLink.
	 * @param value an URI
	 */
	public void writeInClusterLink(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_IN_CLUSTER_LINK, URI.class, value);
	}


	/**
	 * Read the Indicates the service that has produced automatically or
	 *  semi-automatically the resource. 
	 * 
	 * @return an URI
	 */
	public Value<URI> readProducedBy(){
		return applyOperator(Operator.READ, null,URI, IS_PRODUCED_BY, URI.class, null);
	}


	/**
	 * Write the Indicates the service that has produced automatically or
	 *  semi-automatically the resource.
	 * @param value an URI
	 */
	public void writeProducedBy(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, IS_PRODUCED_BY, URI.class, value);
	}


	/**
	 * Read the Refers to a given URI defining an instance of a recognised named entities . 
	 * 
	 * @return an URI
	 */
	public Value<URI> readRefersTo(){
		return applyOperator(Operator.READ, null,URI, REFERS_TO, URI.class, null);
	}


	/**
	 * Write the Refers to a given URI defining an instance of a recognised named entities .
	 * @param value an URI
	 */
	public void writeRefersTo(URI value){
		applyOperator(Operator.WRITE, PREFIX,URI, REFERS_TO, URI.class, value);
	}


	protected static final String CAN_BE_IGNORED = "canBeIgnored";
	protected static final String HAS_CLUSTER_PROXIMITY = "hasClusterProximity";
	protected static final String HAS_GATHERING_DATE = "hasGatheringDate";
	protected static final String HAS_ORIGINAL_FILE_NAME = "hasOriginalFileName";
	protected static final String HAS_ORIGINAL_FILE_SIZE = "hasOriginalFileSize";
	protected static final String HAS_STEM = "hasStem";
	protected static final String IS_CANDIDATE = "isCandidate";
	protected static final String IS_EXPOSED_AS = "isExposedAs";
	protected static final String HAS_BEEN_UPDATED_BY = "hasBeenUpdatedBy";
	protected static final String HAS_NATIVE_CONTENT = "hasNativeContent";
	protected static final String HAS_NORMALISED_CONTENT = "hasNormalisedContent";
	protected static final String IS_COMPOSED_BY_CLUSTER = "isComposedByCluster";
	protected static final String IS_GENERATED_FROM = "isGeneratedFrom";
	protected static final String IS_IN_CLUSTER = "isInCluster";
	protected static final String IS_IN_CLUSTER_LINK = "isInClusterLink";
	protected static final String IS_PRODUCED_BY = "isProducedBy";
	protected static final String REFERS_TO = "refersTo";

}

