package org.ow2.weblab.rdf;

/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2010 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.LinkedList;
import java.util.List;

public class OWL2JavaWebLabAnnotator {

	public static final String WRITE = "applyOperator(Operator.WRITE, !!PREFIX!!,!!URI!!, !!FIELD!!, !!CLASS!!.class, value);";
	public static final String READ = "return applyOperator(Operator.READ, null,!!URI!!, !!FIELD!!, !!CLASS!!.class, null);";
	//"readProperty(!!URI!!, !!FIELD!!, !!CLASS!!.class);";
	private static final String FIRST_VALUE_COMMENT = "";

	//		    "This function will only return the first value even if several are defined.\n" +
	//			"To retrieve all other values, we provide helpers optimized to manage those accesses.\n" +
	//			"@see(http://weblab-project.org/documentation/wiki/index.php?title=Helpers_Documentation)";

	public static void main(String[] args) throws IOException {

		/**
		 * It will build a Java Annotator class extended from AbstractAnnotator
		 */
		if (args.length != 1) {
			showUsage("");
		}


		//		String ontology = "schemas/weblab_retrieval.owl";
		//		String ontology = "schemas/weblab_processing.owl";
		//		String ontology = "src/test/resources/foaf.owl";
		//		String ontology = "src/test/resources/dcelements.rdf";
		//		String ontology = "src/test/resources/little_geonames.owl";
		//		String ontology = "src/test/resources/dcterms.rdf";

		String ontology = args[0];
		if (ontology == null) {
			showUsage("Ontology can not be null.");
		}
		File f = new File(ontology);

		if (f.isDirectory()) {
			showUsage(f.getAbsolutePath() + " should be an ontology not a directory.");
		}

		if (!f.isFile()) {
			showUsage(f.getAbsolutePath() + " should be an ontology file.");
		}

		if (!f.canRead()) {
			showUsage(f.getAbsolutePath() + " should be readable, change read rights on the file.");
		}
		f = null;

		StringWriter sw = new StringWriter();

		InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream(ontology);
		if (is == null) {
			is = new FileInputStream(new File(ontology));
		}
		if (is == null) {
			System.err.println("erf");
		} else {
			BufferedInputStream bis = new BufferedInputStream(is);
			int i = bis.read();
			while (i != -1) {
				sw.write(i);
				i = bis.read();
			}
			bis.close();

		}

		String owl = sw.toString();
		sw = null;

		writeAnnotatorFrom(owl);

	}

	private static void showUsage(String message) {
		System.out.println(message);
		System.out.println("Usage: OWL2JavaWebLabAnnotator path/to/the/owl.or.rdf");
		System.exit(0);
	}

	private static void writeAnnotatorFrom(String owl) throws IOException {
		String fieldBaseURI = "URI";
		String fieldPrefixURI = "PREFIX";
		String baseURI = baseURI(owl);
		String prefixURI = "w" + prefixFrom(baseURI);//

		String ontoComment = ontoComment(owl);

		String className = prefixURI.substring(0, 2).toUpperCase() + prefixURI.substring(2) + "Annotator";
		String packageName = pakageFrom(baseURI);

		FileWriter fw = new FileWriter(new File(className + ".java"));
		fw.write("package " + packageName + ";\n\n");
		fw.write("import java.net.URI;\n");
		fw.write("import java.util.Date;\n");
		fw.write("import org.ow2.weblab.rdf.Value;\n");
		fw.write("import org.ow2.weblab.rdf.XMLParser;\n");
		fw.write("import org.ow2.weblab.core.model.*;\n");
		fw.write("import org.ow2.weblab.core.annotator.*;\n\n");

		fw.write(ontoComment + "public class " + className + " extends BaseAnnotator {\n\n");

		fw.write("\tprotected static final String PREFIX = \"" + prefixURI + "\";\n");
		fw.write("\tprotected static final URI URI = XMLParser.buildURI(\"" + baseURI + "\");\n\n");
		fw.write("\tpublic " + className + "(URI subject, PieceOfKnowledge pieceOfKnowledge){\n");
		fw.write("\t\tsuper(subject,pieceOfKnowledge);\n");
		fw.write("\t}\n\n");


		fw.write("\tpublic " + className + "(Resource resource){\n");
		fw.write("\t\tsuper(resource);\n");
		fw.write("\t}\n\n");

		buildContent(owl, fieldPrefixURI, fieldBaseURI, fw);

		fw.write("\n}\n\n");

		fw.close();
	}

	private static String prefixFrom(String baseURI) {
		int l = baseURI.lastIndexOf('/', baseURI.length() - 2);
		String prefix = baseURI.substring(l + 1, baseURI.length() - 1);
		if (prefix.equalsIgnoreCase("ontology") || prefix.matches("\\d+(\\.\\d+)*")) {
			int s = baseURI.lastIndexOf('/', l - 1);
			prefix = baseURI.substring(s + 1, l);
			prefix = prefix.replaceAll("(www\\.|\\.(org|net|com|fr))", "");
		}
		return prefix;
	}

	private static String pakageFrom(String baseURI) {
		String[] split = baseURI.split("/");
		StringBuffer res = new StringBuffer();
		for (String s : split) {
			if (s.matches(".*\\..*")) {
				String[] parts = s.split("\\.");
				String sr = "";
				for (String p : parts) {
					sr = p + "." + sr;
				}
				res.append("." + sr);
			} else {
				res.append("." + s);
			}
		}
		return res.substring(8, res.length() - 1).replaceAll("\\.+", ".").replaceAll("(-|\\(|\\))", "_");
	}

	private static void buildContent(String owl, String fieldPrefixURI, String fieldBaseURI, FileWriter fw) throws IOException {
		String datatype = "DatatypeProperty";
		String objecttype = "ObjectProperty";
		String rdfproperty = "Property";
		String rdfdescription = "Description";
		List<String> fields = new LinkedList<String>();

		String[] datas = owl.split("<owl:" + datatype + " rdf:about=");
		if (datas.length != 1) {
			for (String data : datas) {
				String property = getPropertyFrom(data);
				if (!property.startsWith("?") && data.indexOf("rdfs:range") != -1) {
					String comment = getCommentFrom(data);
					String type = getTypeFrom(data);
					fields.add(printMethods(fieldPrefixURI, fieldBaseURI, cleanProperty(property), type, comment, fw));
				}
			}
		}


		String[] objects = owl.split("<owl:" + objecttype + " rdf:about=");
		if (objects.length != 1) {
			for (String data : objects) {
				String property = getPropertyFrom(data);
				if (!property.startsWith("?") && data.indexOf("rdfs:range") != -1) {

					String comment = getCommentFrom(data);
					String type = getTypeFrom(data);
					fields.add(printMethods(fieldPrefixURI, fieldBaseURI, property, type, comment, fw));
				}
			}
		}

		String[] rdfproperties = owl.split("<rdf:" + rdfproperty + " rdf:about=");
		if (rdfproperties.length != 1) {
			for (String data : rdfproperties) {
				String property = getPropertyFrom(data);
				if (!property.startsWith("?") && !property.startsWith("!")) {//&& data.indexOf("rdfs:range") != -1) {

					String comment = getCommentFrom(data);
					String type = getTypeFrom(data);
					fields.add(printMethods(fieldPrefixURI, fieldBaseURI, cleanProperty(property), type, comment, fw));
				}
			}
		}

		String[] rdfdescriptions = owl.split("<rdf:" + rdfdescription + " rdf:about=");
		if (rdfdescriptions.length != 1) {
			for (String data : rdfdescriptions) {
				String property = getPropertyFrom(data);
				if (!property.startsWith("?") && !property.startsWith("!") && !property.equals("")) {//&& data.indexOf("rdfs:range") != -1) {

					String comment = getCommentFrom(data);
					String type = getTypeFrom(data);
					fields.add(printMethods(fieldPrefixURI, fieldBaseURI, cleanProperty(property), type, comment, fw));
				}
			}
		}


		for (String field : fields) {
			fw.write(interpret(field, owl) + "\n");
		}
	}


	private static String cleanProperty(String property) {
		property = property.replaceAll("(\\.|-)", "_");
		if (property.indexOf("\"") != -1) {
			int e = property.indexOf("\"");
			return property.substring(0, e);
		}
		return property;
	}

	private static String interpret(String field, String owl) {
		int s = field.indexOf('&');
		int e = field.indexOf(';');
		if (s != -1 && e != -1) {
			return field.substring(0, s) + getEntityValue(field.substring(s + 1, e), owl) + field.substring(e + 1, field.length());
		}
		return field;
	}

	private static String getEntityValue(String entity, String owl) {
		int ents = owl.indexOf("<!ENTITY " + entity);
		int ente = owl.indexOf("\"", ents + 11 + entity.length());
		return owl.substring(ents + 11 + entity.length(), ente);
	}

	private static String baseURI(String owl) {
		int b = owl.indexOf("xml:base=\"");
		String value = "";
		if (b != -1) {
			int end = owl.indexOf("\"", b + 10);
			value = owl.substring(b + 10, end);
			if (value.startsWith("&")) {
				int pe = value.indexOf(";");
				String entity = value.substring(1, pe);
				value = getEntityValue(entity, owl) + value.substring(pe + 1, value.length());
			}
		}

		int s = owl.indexOf("<owl:Ontology rdf:about=\"");
		String about = "";
		if (s != -1) {
			int e = owl.indexOf("\"", s + 25);
			about = owl.substring(s + 25, e);
		} else {
			s = owl.indexOf("<rdf:Description rdf:about=");
			int e = owl.indexOf("\">", s + 28);
			about = owl.substring(s + 28, e);
		}


		if (!about.equals("") && !about.equalsIgnoreCase(value) && about.startsWith("http")) {
			value = about;
		}

		if (!value.endsWith("#") && !value.endsWith("/")) {
			value += "#";
		}

		return value;
	}

	private static String ontoComment(String owl) {
		int o = owl.indexOf("<owl:Ontology");
		String data = "";
		if (o != -1) {
			data = owl.substring(o);
		}

		return generateJavaComment(getCommentFrom(data), null, null).replaceAll("\t", " ");
	}


	private static String getPropertyFrom(String data) {
		int fin = data.indexOf(">") - 1;
		int s = data.substring(0, fin).lastIndexOf("/");
		s = s == -1 ? 1 : s + 1;
		String property = data.substring(s, fin);

		// search for range
		if (property.startsWith("#")) {
			property = property.substring(1);
		}

		return property;
	}


	private static String getTypeFrom(String data) {
		String[] rs = data.split("rdfs:range");
		if (rs.length < 2) {
			return "string";
		}
		String r = rs[1];
		int finr = r.indexOf(">");
		int sr = r.indexOf(";") + 1;
		String type = "Resource";
		if (finr != 0) {
			type = r.substring(sr, finr - 2).replaceAll("\"", "");
			if (type.trim().startsWith("rdf:resource=")) {
				if (type.trim().startsWith("rdf:resource=\"#")) {
					type = type.trim().substring(16);
				} else {
					type = type.trim().substring(14);
				}
			}
		}
		return type;
	}

	private static String getCommentFrom(String data) {
		String[] com = data.split("comment");
		if (com.length < 2) {
			return "";
		}
		String c = com[1];
		int sc = c.indexOf(">") + 1;
		int ec = c.indexOf("<");
		if (ec == -1 || sc == 0) {
			return "";
		}
		return c.substring(sc, ec).trim();
	}

	private static String printMethods(String prefixURI, String baseURI, String value, String r, String comment, FileWriter fw) throws IOException {
		String type = retrieveType(r);
		String field = generateStaticField(value);

		String cleanedValue = value.replaceAll("&", "").replace(';', '_');

		String getter = generateGetter(baseURI, cleanedValue, field, type, comment);
		String setter = generateSetter(prefixURI, baseURI, cleanedValue, field, type, comment);

		fw.write(getter + "\n");
		fw.write(setter + "\n");

		//		System.out.println(getter);

		//		System.out.println(setter);

		return "\tprotected static final String " + field + " = \"" + value + "\";";

	}

	private static String generateGetter(String baseURI, String value, String field, String type, String comment) {

		if (value.startsWith("has")) {
			value = "read" + value.substring(3, 4).toUpperCase() + value.substring(4);
		} else if (value.startsWith("is")) {
			value = "read" + value.substring(2, 3).toUpperCase() + value.substring(3);
		} else {
			value = "read" + value.substring(0, 1).toUpperCase() + value.substring(1);
		}

		String res = "\tpublic Value<" + type + "> " + value + "(){\n" + "\t\t" + READ.replaceFirst("!!URI!!", baseURI).replaceFirst("!!FIELD!!", field).replaceFirst("!!CLASS!!", type) + "\n"
				+ "\t}\n\n";
		return generateJavaComment("Read the " + comment + " \n" + FIRST_VALUE_COMMENT, null, type) + res;
	}

	private static String generateSetter(String prefixURI, String baseURI, String value, String field, String type, String comment) {
		if (value.startsWith("has")) {
			value = "write" + value.substring(3, 4).toUpperCase() + value.substring(4);
		} else if (value.startsWith("is")) {
			value = "write" + value.substring(2, 3).toUpperCase() + value.substring(3);
		} else {
			value = "write" + value.substring(0, 1).toUpperCase() + value.substring(1);
		}

		String res = "\tpublic void " + value + "(" + type + " value){\n" + "\t\t"
				+ WRITE.replaceFirst("!!URI!!", baseURI).replaceFirst("!!PREFIX!!", prefixURI).replaceFirst("!!FIELD!!", field).replaceFirst("!!CLASS!!", type) + "\n" + "\t}\n\n";
		return generateJavaComment("Write the " + comment, type, null) + res;
	}

	private static String generateJavaComment(String comment, String param, String ret) {
		String res = "\t/**\n";
		res += "\t * " + comment.replaceAll("( |\\t)+", " ").replaceAll("\\n", "\n\t * ");
		if (param != null) {
			String des = desFrom(param);
			res += "\n\t * @param value " + des + param;
		}
		if (ret != null) {
			String des = desFrom(ret);
			res += "\n\t * @return " + des + ret;
		}
		return res + "\n\t */\n";
	}

	private static String desFrom(String param) {
		switch (param.toLowerCase().charAt(0)) {
			case 'a':
			case 'e':
			case 'i':
			case 'o':
			case 'u':
			case 'y':
				return "an ";
			default:
				break;
		}
		return "a ";
	}

	private static String generateStaticField(String value) {
		StringBuffer res = new StringBuffer();
		for (int i = 0; i < value.length(); i++) {
			char c = value.charAt(i);
			if (!(c + "").toLowerCase().equals("" + c)) {
				res.append("_");
			}
			res.append(c);
		}
		return res.toString().replaceAll("(&|;)", "").toUpperCase();
	}

	private static String retrieveType(String r) {
		if (r.matches("int")) {
			return "Integer";
		} else if (r.matches("long|double|boolean|string|date")) {
			return ((char) (r.charAt(0) + ('A' - 'a'))) + r.substring(1);
		} else if (r.matches("(.*Thing|.*Literal)")) {
			return "String";
		} else {
			return "URI";
		}
	}

}
